function object = flt_stats_reader( inputs, mode, start_inds )

% object = flt_stats_reader( { {op1, var1}, ..., {opN, varN} } mode )
% object = flt_stats_reader( {op, var}, mode )
% object = flt_stats_reader( op, mode )
%
% op-var pair(s)   these name pairs define the input sources.
%                  a var field can also be '' (or omitted in the last
%                  form). This causes the filter to act as an iteration
%                  counter.
% mode            'calls' or 'iters':
%                   calls   one value is read for each completed call.
%                           (see the stats constructor)
%                   iters   one value is read for each iteration.
%                           (see the stats constructor)
%                           NOTE: remember that stats counts an operation
%                                 start also as an iteration. keep this is
%                                 in mind when constructing filter and
%                                 trigger chains!
%
% This filter reads sequentially from STATS, starting from first new record
% after set_enabled() has been called. An input can be a variable or it can
% be an iteration counter. In the first case, all new values for the
% requested variable are returned sequentially (NaN's are skipped). In the
% second case, an undefined value is read on the operation's each call or real
% iteration (the measure('start') iteration is ignored), depending on 'mode'.
%
% The filter output from pop() will be a vector with same length equal to
% the number of input sources, each element containing the next scalar output
% of the corresponding source. If an operation doesn't have any new results
% at the moment, then the corresponding element will contain a NaN.
%
% Mainly for debugging reasons, there is also an optional argument
% 'start_inds', which specifies the read start rows.
%
% Example:
%   flt = flt_stats_reader( {{'Reinit_model', 'efficiency'}, ...
%                            {'Improve_model', 'efficiency'}}, 'calls' );
%   flt = set_active( flt, 1 );
%   ...
%   [flt, output] = pop( flt );
%   while any(~isnan(output))
%     ...
%     [flt, output] = pop( flt );
%   end


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global STATS;


% define the class so that matlab recognizes it when loading objects
if nargin == 0
  this.mode         = [];
  this.nchannels    = [];
  this.op_names     = [];
  this.var_names    = [];
  this.last_entries = [];
  this.nan_vals     = [];
  object = class( this, 'flt_stats_reader', flt );
  return;
end


% parse inputs into op_names and var_names
if iscell(inputs)
  if iscell(inputs{1})
    temp      = cat(1,inputs{:});
    op_names  = temp(:,1);
    var_names = temp(:,2);
  else
    op_names  = {inputs{1}};
    var_names = {inputs{2}};
  end
else
  op_names  = {inputs};
  var_names = {''};
end
var_names(strcmp(var_names, '')) = {'efficiency'};

if exist('start_inds', 'var') start_inds=start_inds-1; else start_inds=[]; end


this.mode         = mode;
this.nchannels    = length(op_names);
this.op_names     = op_names;
this.var_names    = var_names;
this.last_entries = start_inds;
this.nan_vals     = NaN * zeros( 1, this.nchannels );

object = class( this, 'flt_stats_reader', flt( this.nchannels ) );
return;
