function obj = stats()

% obj = stats
%
% Creates a new 'stats' object, which can be used to measure, store and
% process function execution times, cost improvements etc..
% The stats object contains also a cputimer, see the cputimer() method.
% Initially the created object is in inactive state.
%
%
% Measuring:
%
% Progress of various operations can be recorded by first creating a stats
% object, activating it and its cputimer, then adding the target operations
% with add_operation() and finally recording the progress with measure()
% method calls.
%
% Measurements consist from operation calls and operation
% iterations. Calling measure('start') marks a call as been started,
% each measure('checkpoint') records one iteration, and finally
% measure('stop') marks the call as finished.
%
% More information about measuring can be found from the help of measure()
% method.
%
%
% Viewing or using the results:
%
% Recorded progress of various operations can be read directly with methods
% get_res() and get_value(). The first reads formatted result tables and the
% other reads only one value or row at a time.
%
% Recorded progress information can also be used as an input to a
% trigger/filter chain with filter flt_stats_reader.
%
% A general summary from one or more stats object can be viewed with
% function info().
%
%
% Viewing results at iteration level / call level:
%
% The results can be viewed at iteration level or call level. The level is
% chosen with 'calls' and 'iters' flags that many methods accept.
% Internally all results are stored like they are seen at iteration level,
% and the call level results are always generated on the fly when requested
% (==> iter level == fast, call level == slow).
% Iteration level:
%   For every operation call there is a 'start' row (situation when
%   measure('start') was called) followed by some iteration rows (situation
%   on each measure('checkpoint') call). Row groups (areas) resulted from
%   consecutive operation calls are one after another in the result
%   matrix. Area bounds can be viewed with get_areas() method.
%   NOTE: An area contains always one more row than there were iterations,
%         and this may cause some trouble when constructing filter and
%         trigger chains originating from an iteration level stats reader
%         filter. See flt_stats_reader's help for one solution.
%   NOTE2: The efficiency value has one special case: if on some iteration
%          the cputimer delta is zero but cost delta is not, then a zero
%          efficiency is recorded instead of a NaN! (see measure() method's
%          comments for the reason)
% Call level:
%   For every operation call there is a single row, which contains the
%   final values substracted by the initial values of all
%   variables. Efficiency is an exception: it is recomputed to reflect the
%   call's average efficiency. Also some extra variables are added, see
%   method get_value for more information.
%
%
% Variables that are always recorded:
%   'cputimer'
%   'total cost'
%   'efficiency'
% Some extra variables are available when viewing the results at call
% level. More extra variables can be defined for individual operations when
% adding them with add_operation().
%
% Note: 'cputimer' and 'efficiency' variables depend on the cputimer, which
%       must be started manually! (see the cputimer() method)


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.



% The indices of variables cputimer, total cost and efficiency are
% hardcoded in some methods!
%
% All methods use the operation or variable name instead of its index. This
% causes quite a lot of repeated lookups in some methods..
% (ok, profiling shows that this is not so time consuming after all)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


this.active = 0;
this.active_stack = {};

this.statevars.dummy = [];
this.startstates = {};

this.statevars.internal_quiet       = false;
this.statevars.internal_addr_cursor = ...
    environment( 'has_feature', 'addressable_cursor' );

this.cputimer.active = 0;
this.cputimer.elapsed = 0.0;

this.result_names = {};
this.results      = {};
this.var_names    = {};
this.print_vars   = {};
this.areas        = {};
this.print_indent = 0;

obj = class( this, 'stats' );
return;
