function [Summary, All] = compute_batch_summary(Results, Mode, RefGroups)

% [Summary, All] = compute_batch_summary(Results, Mode, [RefGroups])
%
% INPUT ARGUMENTS
%
%  Results       cell array of results
%  Mode          one of the following: 'overall', 'groups', 'compare',
%                'var_pairs'
%  [RefGroups]   reference groups; used by compare mode only
%
% OUTPUT ARGUMENTS
%
%  Summary       summary information, content depends on mode
%  All           a vector which contains indices for the summary data 
%
% PURPOSE
%
%  Helper function for ivga_show.


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin == 0, help(mfilename); return; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nruns = max(size(Results));

if ~iscell(Results)
  Results = {Results};
end
  
nvars = length(cat(2, Results{1}.group_models.member_variables));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch(Mode)

 case 'overall'

  NGroups           = zeros(nruns, 1);
  NVars             = cell(nruns, 1);
  NCentroids_total  = zeros(nruns, 1);
  NCentroids_models = cell(nruns, 1);
  Costs_total       = zeros(nruns, 1);
  Costs_models      = cell(nruns, 1);
  
  for i = 1:nruns
    
    NGroups(i) = max(size(Results{i}.grouping));
    
    for j = 1:NGroups(i)
      NCentroids_models{i}(j) = get_ncentroids(Results{i}.group_models(j).internal);
      NCentroids_total(i)     = NCentroids_total(i) + NCentroids_models{i}(j);
      
      Costs_models{i}(j)      = get_cost(Results{i}.group_models(j).internal);
      Costs_total(i)          = Costs_total(i) + Costs_models{i}(j);
      
      NVars{i}(j) = length(Results{i}.grouping{j});
    end  

  end

  Summary = struct('NGroups', {NGroups}, 'NVars', {NVars}, ...
                   'NCentroids_models', {NCentroids_models}, ...
                   'NCentroids_total', {NCentroids_total}, ...
                   'Costs_models', {Costs_models}, ...
                   'Costs_total', {Costs_total});
    
  All = 1:nruns;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 case 'groups'

  ngroups         = 0;
  Groups          = cell(0);
  GroupCosts      = cell(0);
  GroupNCentroids = cell(0);

  for i = 1:nruns

    for j = 1:length(Results{i}.grouping)

      Current_group = Results{i}.group_models(j).member_variables;
      cost          = get_cost(Results{i}.group_models(j).internal);
      centroids     = get_ncentroids(Results{i}.group_models(j).internal);
    
      k = 1;

      while (k <= ngroups)

        Comp = Groups{k};

        if (length(Comp) == length(Current_group) & Comp == Current_group)
          GroupFreqs(k)      = GroupFreqs(k) + 1;
          GroupCosts{k}      = [GroupCosts{k} cost];
          GroupNCentroids{k} = [GroupNCentroids{k} centroids];
          break;
        else
          k = k + 1;
        end
      end
      
      if (k > ngroups)
        ngroups                  = ngroups + 1;
        GroupFreqs(ngroups)      = 1;
        Groups{ngroups}          = Current_group;
        GroupCosts{ngroups}      = [cost];
        GroupNCentroids{ngroups} = [centroids];
      end
        
    end
    
  end

  [Dummy Ind] = sort(-GroupFreqs, 2); % sort GroupsFreqs in descending order
  GroupsFreqs      = GroupFreqs(Ind); %
  Groups           = Groups(Ind);
  GroupCosts       = GroupCosts(Ind);
  GroupNCentroids  = GroupNCentroids(Ind);
  
  Summary = struct('Groups', {Groups}, 'GroupFreqs', {GroupFreqs}, ...
                   'GroupCosts', {GroupCosts}, ...
                   'GroupNCentroids', {GroupNCentroids});

  All = 1:ngroups;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 case 'compare'

  nrefgroups   = length(RefGroups);
  Matches      = zeros(nrefgroups, 1);
  ExactMatches = zeros(nrefgroups, 1);

  for i = 1:nrefgroups
    [Matches(i) ExactMatches(i)] = compute_matches(Results, nruns, ...
                                                   RefGroups{i});
        
  end

  Matches      = 100 * Matches / nruns;
  ExactMatches = 100 * ExactMatches / nruns;
 
  Summary = struct('Matches', Matches, 'ExactMatches', ExactMatches);

  All = nrefgroups;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
 case 'var_pairs'

  PairOccurrencies = zeros(nvars, nvars);
  
  for i = 1:nvars
    for j = i:nvars
      PairOccurrencies(i, j) = compute_matches(Results, nruns, [i j]);
      PairOccurrencies(j, i) = PairOccurrencies(i, j);
    end
  end
  
  PairOccurrencies = 100 * PairOccurrencies / nruns;
  
  Summary = struct('PairOccurrencies', PairOccurrencies);

  All = 1:nvars;
  
end

Summary.nruns = nruns;
Summary.nvars = nvars;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [matches, exactMatches] = compute_matches(Results, nruns, RefGroup)

matches      = 0;
exactMatches = 0;
refgrouplen  = length(RefGroup);

for i = 1:nruns
  
  Groups = Results{i}.grouping;
  
  for j = 1:length(Groups)
    
    nmatches = sum(ismember(RefGroup, Groups{j}));
    if (nmatches == refgrouplen)
      matches = matches + 1;
      if (nmatches == length(Groups{j}))
        exactMatches = exactMatches + 1;
      end
    end
  end
      
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

